@@ -287,57 +287,57 @@ checkWarning <- function(expr, expected=NULL, msg=NULL){
 
 #' Make Vignette for Unit Tests
 #' 
+#' Builds a vignette for unit tests in a package using the \code{\link{utest}} 
+#' and a template vignette file. 
+#' 
 #' @param pkg Package name
 #' @param file Output file (.Rnw, .tex, or .pdf)
+#' @param ... extra arguments passed to \code{\link{utest}}.
+#' @param check logical that indactes the cal was made from R CMD check, in which case the vignette
+#' is updated only if results of unit tests can be found in the unit test output directory, 
+#' where they would have been generated by \code{\link{utest}}.
+#' 
 #' @return Result of running unit test suite
 #'  
 #' @export
 #' 
-makeUnitVignette <- function(pkg, file=paste(pkg, '-unitTests.pdf', sep='')){
+makeUnitVignette <- function(pkg, file=paste(pkg, '-unitTests.pdf', sep=''), ..., check=FALSE){
 	
+	package <- pkg
+	pkg <- sub("^package:", "", pkg)
 	# generate the vignette for unit test on exit
 	if( !is.null(file) )
-		on.exit( writeUnitVignette(pkg, file) )
+		on.exit( writeUnitVignette(pkg, file, check=check) )
 	# load this package
-	require( pkg, character.only = TRUE )
-	
-	## load RUnit
-	requireRUnit("Make unit test vignette")
-	if( file.exists( "unitTests-results" ) ){
-		file.remove("unitTests-results") 
-	}
-	dir.create( "unitTests-results" )
-	
-	path <- system.file("unitTests", package = pkg)
-	testSuite <- defineTestSuite(name=paste(pkg, "unit testing")
-			, dirs = path
-			, rngKind = "default",
-			, rngNormalKind = "default")
-	tests <- runTestSuite(testSuite)
-	printHTMLProtocol(tests, fileName= sprintf( "unitTests-results/%s-unitTests.html" , pkg ) )
-	printTextProtocol(tests, fileName= sprintf( "unitTests-results/%s-unitTests.txt"  , pkg ) )
-	
-	# check for errors
-	err <- getErrors(tests)
-	errMsg <- NULL
-	if( err$nFail > 0) {
-		errMsg <- c(errMsg, sprintf( "unit test problems: %d failures\n", err$nFail))
-	}
-	if( err$nErr > 0) {
-		errMsg <- c(errMsg, sprintf( "unit test problems: %d errors\n", err$nErr))
+	if( !require(pkg, character.only = TRUE ) ){
+		stop("Could not load package '", pkg, "' for testing [libPath= ", str_out(.libPaths(), Inf), "]")
 	}
-	# stop if any failure or error occured
-	if( length(errMsg) > 0L )
-		stop(errMsg)
 	
-	# copy result in tmp directory if possible
-	if( file.exists( "/tmp" ) ){
-		file.copy( sprintf( "unitTests-results/%s-unitTests.txt" , pkg ) , "/tmp", overwrite = TRUE )
-		file.copy( sprintf( "unitTests-results/%s-unitTests.html", pkg ) , "/tmp", overwrite = TRUE )
+	# run unit tests if not check or if the test results are not there (e.g., R CMD build) 
+	if( !check || !is.dir(utestPath(package=package)) ){
+		
+		# run unit tests
+		tests <- utest(package, ...)
+		
+		# check for errors
+		err <- getErrors(tests)
+		errMsg <- NULL
+		if( err$nFail > 0) {
+			errMsg <- c(errMsg, sprintf( "unit test problems: %d failures\n", err$nFail))
+		}
+		if( err$nErr > 0) {
+			errMsg <- c(errMsg, sprintf( "unit test problems: %d errors\n", err$nErr))
+		}
+		# stop if any failure or error occured
+		if( length(errMsg) > 0L )
+			stop(errMsg)
+		
+		# return result of unit test suite
+		err
+	}else{
+		# do nothing: tests should have been already run by R CMD check
 	}
 	
-	# return result of unit test suite
-	err
 }
 
 #' Writes Unit Tests Vignette 
@@ -346,11 +346,15 @@ makeUnitVignette <- function(pkg, file=paste(pkg, '-unitTests.pdf', sep='')){
 #' 
 #' @param pkg Package name
 #' @param file Output Sweave (.Rnw) file
+#' @param results result file or output character vector
+#' @param check logical that indactes the cal was made from R CMD check, 
+#' in which case the vignette is updated only if results of unit tests can 
+#' be found in the unit test output directory, where they would have been 
+#' generated by \code{\link{utest}}.
 #' 
 #' @export
 #' 
-writeUnitVignette <- function(pkg, file){
-	
+writeUnitVignette <- function(pkg, file, results=NULL, check=FALSE){
 	Rnw.template <- 
 "
 \\documentclass[10pt]{article}
@@ -371,7 +375,7 @@ authors <- packageDescription(pkg)$Author
 
 \\usepackage[colorlinks]{hyperref}
 \\author{\\Sexpr{authors}}
-\\title{\\texttt{\\Sexpr{pkg}}: Unit testing results}
+\\title{\\texttt{\\Sexpr{pkg}}: Unit testing results@resNote@}
 \\date{\\texttt{\\Sexpr{pkg}} version \\Sexpr{prettyVersion} as of \\Sexpr{prettyDate}}
 \\begin{document}
 \\maketitle
@@ -385,15 +389,22 @@ authors <- packageDescription(pkg)$Author
 
 \\end{document}
 "
-	# load the results of the unit tests
-	results <- file.path('unitTests-results', paste(pkg, '-unitTests.txt', sep=''))
-	results <- 
-	if( file.exists( results ) ){
-		paste(readLines(results), collapse="\n")
-	} else{
-		'unit test results not available'
+	# default is to load the unit test results from the global output directory
+	if( is.null(results) ){
+		upath <- utestPath(package=pkg)
+		results <- list.files(upath, pattern="\\.txt$", full.names=TRUE)
+		if( !length(results) )
+			results <- str_c('Could not find any unit test result in "', upath, '"')
 	}
 	
+	if( is.file(results[1L]) ){
+		resFile <- results[1L]
+		results <- readLines(resFile)
+	}else{
+		resFile <- NULL
+	}
+	results <- paste(results, collapse="\n")
+	
 	# substitute template variables
 	contents <- Rnw.template
 	# package name
@@ -402,6 +413,14 @@ authors <- packageDescription(pkg)$Author
 	contents <-	gsub("@results@", results, contents)
 	# session info (as when calling this function)
 	contents <-	gsub("@sessionInfo@", gsub("\\", "\\\\", paste(toLatex(sessionInfo()), collapse="\n"), fixed=TRUE), contents)
+	# note on how tests were performed
+	resnote <- str_c("\\footnote{Vignette computed ", if( check ) ' via R CMD check/build ', ' on ', date(),"}")
+	if( check ){ 
+		# add path to included file if compiled from R CMD check (for debug purposes)
+		lfile <- gsub("([_$])", "\\\\\\1", resFile)
+		resnote <- str_c(resnote, " \\footnote{File: '", lfile, "'}")
+	}
+	contents <-	gsub("@resNote@", gsub("\\", "\\\\", resnote, fixed=TRUE), contents)
 	
 	fileext <- toupper(file_extension(file))
 	fileext <- charmatch(fileext, c('RNW', 'TEX', 'PDF'))
@@ -421,7 +440,11 @@ authors <- packageDescription(pkg)$Author
 	if( fileext == 2L ) return()
 	
 	# Run texi2dvi tex file
-	tools::texi2dvi(fileTEX, pdf = TRUE, clean = TRUE )
+	res <- tools::texi2dvi(fileTEX, pdf = TRUE, clean = TRUE )
+	
+	# copy file in main check directory
+	if( check )	file.copy(filePDF, '../../..')
+	res
 }
 
 # Unit test frameworks data
@@ -636,11 +659,14 @@ setMethod('utest', 'function',
 #' @param testdir directory where to look for the test files
 #' @param framework unit test framework
 #' @param quiet a logical that indicates if the tests should be run silently
+#' @param lib.loc path to a library where installed packages are searched for.
+#' Used is of the form \code{x='package:*'}.
 #'  
 setMethod('utest', 'character', 
 		function(x, filter="^runit.+\\.[rR]$", fun="^test\\.", ...
 				, testdir='tests', framework=c('RUnit', 'testthat')
-				, quiet = Sys.getenv("RCMDCHECK") != "FALSE"){
+				, quiet = Sys.getenv("RCMDCHECK") != "FALSE"
+				, lib.loc = NULL){
 			
 			cat("#########################\n")
 			#print(system('env'))
@@ -649,7 +675,7 @@ setMethod('utest', 'character',
 					if( grepl("^package:", x) ){# installed package
 						pkg <- sub("^package:", "", x)
 						if( is.null(path <- path.package(pkg, quiet=TRUE)) ){
-							library(pkg, character.only=TRUE)
+							library(pkg, character.only=TRUE, lib.loc=lib.loc)
 							path <- path.package(pkg)
 						}
 						file.path(path, testdir)
@@ -666,13 +692,17 @@ setMethod('utest', 'character',
 			
 			# check that the path exists
 			if( !file.exists(path) ){
-				if( !hasArg(testdir) ){
+				if( !hasArg(testdir) ){ # try another default
+					opath <- path
 					path <- file.path(dirname(path), 'unitTests')
-				}
-				if( !file.exists(path) )
+					if( !file.exists(path) )
+						stop("Could not find any default unit test directory ['", opath, "' nor '", path, "'].")
+				} else {
 					stop("Unit test directory '", path, "' does not exist")
+				}
 			}
 			
+			message("Running unit tests in: '", path, "'")
 			# detect unit test framework: RUnit or testthat?
 			framework <- 
 					if( missing(framework) ) utestFramework(path)
@@ -685,6 +715,14 @@ setMethod('utest', 'character',
 			if( missing(fun) ) fun <- up$fun_pattern
 			
 			# run tests
+			path <- normalizePath(path)
+			# remove/create output directory
+			opath <- utestPath(package=x)
+			if( file.exists( opath ) ){
+				file.remove(opath, recursive=TRUE) 
+			}
+			dir.create(opath, recursive=TRUE)
+			
 			if( is.dir(path) ){ # all tests in a directory
 				if( framework == 'RUnit' ){ # RUnit
 					
@@ -692,7 +730,8 @@ setMethod('utest', 'character',
 					s <- defineTestSuite(x, path
 							, testFileRegexp=filter
 							, testFuncRegexp=fun, ...)
-					utest(s, quiet=quiet)
+					str(s)
+					utest(s, quiet=quiet, outdir=opath)
 					
 				}else if( framework == 'testthat' ){ # testthat
 					
@@ -719,9 +758,14 @@ setMethod('utest', 'character',
 
 setOldClass('RUnitTestSuite')
 #' Runs a RUnit test suite
+#' 
+#' @param outdir output directory
 setMethod('utest', 'RUnitTestSuite',
-	function(x, ..., quiet=FALSE){
+	function(x, ..., quiet=FALSE, outdir=NULL){
 		requireRUnit("Running RUnit test suites")
+		
+		pathReport <- file.path(outdir, str_c("utest.", sub("[:]", "_", x$name)))
+		
 		if( quiet ){
 			suppressWarnings(suppressMessages(out <- capture.output(
 				tests <- runTestSuite(x, ...)
@@ -729,17 +773,16 @@ setMethod('utest', 'RUnitTestSuite',
 		}else 
 			tests <- runTestSuite(x, ...)
 		
-		pathReport <- str_c("utest.", sub("[:]", "_", x$name))
 		## Report to stdout and text files
 		cat("------------------- UNIT TEST SUMMARY ---------------------\n\n")
-		printTextProtocol(tests, showDetails=FALSE)
 		printTextProtocol(tests, showDetails=FALSE,
-				fileName=paste(pathReport, "Summary.txt", sep=""))
+				fileName=paste(pathReport, ".Summary.txt", sep=""))
 		printTextProtocol(tests, showDetails=TRUE,
 				fileName=paste(pathReport, ".txt", sep=""))
 		
 		## Report to HTML file
 		printHTMLProtocol(tests, fileName=paste(pathReport, ".html", sep=""))
+		printHTMLProtocol(tests, fileName=paste(basename(pathReport), ".html", sep=""))
 		
 		## Return stop() to cause R CMD check stop in case of
 		##  - failures i.e. FALSE to unit tests or
@@ -749,5 +792,22 @@ setMethod('utest', 'RUnitTestSuite',
 			stop(paste("\n\nunit testing failed (#test failures: ", tmp$nFail,
 							", #R errors: ",  tmp$nErr, ")\n\n", sep=""))
 		}
+		
+		tests
 	}
-)
\ No newline at end of file
+)
+
+#' Unit Tests Result Directory
+#' 
+#' Returns the path to the directory where the results of unit tests are stored.
+#' This path is used by \code{\link{utest}} to save unit test results, which are 
+#' read by \code{\link{makeUnitVignette}} to update the unit test vignette when 
+#' runnning R CMD check.  
+#' 
+#' @param ... extra arguments passed to \code{\link{packagePath}}, e.g., \code{package}.
+#' 
+#' @export
+utestPath <- function(...){
+	packagePath('tests-results', ...)
+}
+
